************************************************************************
*FUNCTION CGIMAIN
******************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1995
***   Contact: (503) 386-2087  / 76427.2363@compuserve.com
***  Modified: 11/06/95
***  Function: Web Connect Main program used on West Wind Server
***            at Gorge Net.
***
***            Also contains Class Settings which is used
***            to retrieve defaults from CGIMAIN.INI if it exists
************************************************************************
LOCAL lcOldError, loAPI

#INCLUDE FOXPRO.H
#INCLUDE WCONNECT.H

PUBLIC glExitCGIServer, glRecycle, goSettings, oCGIServer
glExitCGIServer=.F.
glRecycle=.F.

DO WHILE !glExitCGIServer
   SET SAFETY OFF
   SET EXACT OFF
   SET TALK OFF
   SET DELETED ON
   SET ESCAPE ON
   SET CPDIALOG OFF
      
   #IF !DEBUGMODE
     SET DEBUG OFF
     SET STATUS BAR OFF
     SET DEVELOP OFF
   #ENDIF
   
   *** Allow shut down by system
   ON SHUTDOWN DO ShutIt IN CGIMAIN.PRG
   
   #IF SHAREWARE
     DO WCONNECT
   #ELSE
     SET PROCEDURE TO CGIServ ADDITIVE
     SET PROCEDURE TO CGI ADDITIVE
     SET PROCEDURE TO HTML ADDITIVE
     SET PROCEDURE TO CGIPROC ADDITIVE
     SET PROCEDURE TO wwUTILS ADDITIVE
   #ENDIF

   *** Add any project paths here
   DO PATH WITH "WWDEMO"
      
   *** Retrieve INI file settings or defaults
   *** NOTE: This is a PUBLIC object!
   oSettings=CREATE("Settings",".\cgimain.ini")

   IF NOT oSettings.lDebugMode
     ON ERROR DO ErrorHandler   && Shut down and restart
     ON ERROR DO ErrorHandler WITH ;
              ERROR(),MESSAGE(),MESSAGE(1),SYS(16),LINENO()
   ENDIF

   oCGIServer=CREATE("wwCGIServer","Process")
   
   IF TYPE("oCGIServer")#"O"
      =MessageBox("Unable to load the CGI Request Server",;
         MB_ICONEXCLAMATION,"Web Connection Error")
      RETURN
   ENDIF
   
   *** You can also force the CGI server to process requests
   *** on a remote server, thus scaling the load to
   *** multiple processors. Point at the CGI temp file directory
   *** over the network with this method.
   oCGIServer.SetCGIFilePath(oSettings.cPath) 
   oCGIServer.SetCGIFileTemplate(oSettings.cTemplate)
   oCGIServer.SetForegroundTasking(oSettings.nPriority)
   oCGIServer.SetTimerInterval(250)
   *oCGIServer.SetStatusWindow(.F.)
   oCGIServer.SetLogToFile(oSettings.lLogToFile)
   
   *** This actually puts the server into polling mode
   *** Displays a modal window 
   oCGIServer.show(1)
   
   *** Done
   ON SHUTDOWN
ENDDO

ON ERROR
RELEASE glExitCGIServer

IF glRecycle
   *** Type these into the command window
   *** to clear the program and restart
   KEYBOARD "CLEAR ALL"+CHR(13)+;
            "CLOSE ALL"+CHR(13)+;
            "CLEAR PROG"+CHR(13)+;
            "DO CGIMAIN"+CHR(13)
ENDIF   

RETURN


************************************************************************
FUNCTION Process
******************
***  Modified: 01/24/96
***  Function: This procedure's main purpose is to route incoming
***            CGI requests to individual project PRGs/APPs.
***
***            The URL formatting used is as follows:
***            /cgi-win/wwcgi.dll?project~task~Optional+Parms
***
***            The project 'parameter' is routed to the appropriate
***            program file which actually implements a Process class
***            to respond to CGI requests.
***             
***            Several Maintaince functions are provided directly in
***            this program file: ShowLog, Clear Log and Recycle (Server).
***
***      Pass: loCGI	-	Object containing CGI information
***    Return: .T. to erase Temp File .F. to keep it
************************************************************************
LPARAMETERS loCGI

IF TYPE("loCGI")#"O"
   *** This will cause a server error, but we
   *** can't work without the CGI info
   RETURN .F.
ENDIF
   
*** Optional CGI parameters following EXE name
*** Format: /cgi-win/wwcgi.dll?wwdemo~Project~Method~Other+Parameters
loCGI.aCGIParms()   && Create Array of parameters 
lcParameter=UPPER(loCGI.GetCGIParameter(1))  && Get first Parameter (Project)

*** Set up project types and call external processing programs:
DO CASE

  CASE lcParameter="WWDEMO"
     DO wwDemo with loCGI

*-*  CASE lcParameter="MYCODE"
*-*     DO MyCode with loCGI
     
*-*  CASE lcParameter="WINDANCE"
*-*     DO Windance with loCGI
*-*  CASE lcParameter="WESTWIND"
*-*     DO westwind with loCGI
*-*  CASE lcParameter="USSA"
*-*     DO pnsa with loCGI
*-*  CASE lcParameter="WEBHITS"
*-*     DO Webhits with loCGI


  *** Generic Page Display Routine - Just pass page name on the WWCGI command line
  CASE lcParameter = "DISPLAYFILE" OR lcParameter="SHOWPAGE"
      DO DisplayPage with loCGI, HTMLPAGEPATH
    
  *** SYSTEM TASKS
  CASE lcParameter="SHOWLOG"
     Do ShowLog with loCGI
  CASE lcParameter="SHOWERRORS"
     Do ShowLog with loCGI, .t.
  CASE lcParameter="CLEARLOG"
     Do ClearLog with loCGI
  CASE lcParameter="RUNQUIT"
     loHTML=CREATE("wwHTML",loCGI.GetOutFile())
     loHTML.HTMLError("Visual FoxPro about to shutdown",;
                      "Sucessful operation")
     RELEASE loHTML
     QUIT
  OTHERWISE
     loHTML=CREATE("wwHTML",loCGI.GetOutFile())
     IF TYPE("loHTML")#"O"
        *** This will cause a server error, but we
        *** can't work without the CGI info
        RETURN .F.
     ENDIF
     loHTML.HTMLError("Unhandled CGI Request",;
                  "The server is not setup to handle this type of CGI Request",;
                  loCGI.GetServerAdmin(), loCGI.GetPreviousUrl())
ENDCASE

*** Debug: See what the input and output files look like
* RELEASE LOHTML
* COPY FILE (loCGI.GetContentFile()) TO TEMP.INI  
* COPY FILE (loCGI.GetOutFile()) TO TEMP.HTM  
RETURN



************************************************************************
PROCEDURE DisplayPage
*********************
***  Modified: 04/07/96
***  Function: Generically displays a template page that is displayed
***            with wwHTML::ShowMemoPage
***    Assume: Called with wwcgi.dll?ShowPage~Pagename.wc
***                        wwcgi.dll?DisplayPage~PageName.wc
***    Return: Nothing
*************************************************************************
LPARAMETER loCGI, lcBasePath

lcBasePath=IIF(type("lcBasePath")="C",lcBasePath,"")

  *** Now create a process object and call DisplayFile method
  loCGIProcess=CREATE("wwCGIProcess",loCGI)
  IF TYPE("loCGIProcess")#"O"
     *** All we can do is return...
     WAIT WINDOW NOWAIT "Unable to create CGI Processing object..."
     RETURN .F.
  ENDIF
  loCGIProcess.DisplayFile(loCGIProcess.oCGI.GetCGIParameter(2),;
                           lcBasePath)
RETURN


************************************************************************
PROCEDURE ShowLog
******************
***  Modified: 01/12/96
***  Function: Displays the last 300 records in the CGILOG file.
***      Pass: loCGI object
*************************************************************************
LPARAMETER loCGI, llShowErrorLog

IF !USED("CGILOG")
   USE CGILOG in 0
ENDIF

SELE CGILOG

lnReccount=reccount()

loHTML=CREATE("wwHTML",loCGI.GetOutFile())

loHTML.SetAllowHTMLTables(loCGI.IsNetScape())

IF !llShowErrorLog
   loHTML.HTMLHeader("CGI Access Log Display")
   loHTML.SendLn("<b>CGILOG.DBF Record Count: "+STR(lnReccount)+"</b><BR>")
   
   lnReccount=IIF(lnreccount>MAX_TABLE_CELLS/4-1,MAX_TABLE_CELLS/4-1,lnReccount)
   loHTML.SendLn("Displaying last "+STR(lnreccount)+" Entries <p>")
   
   SELECT time,PADR(MLINE(Request,1),40) AS Request,RemoteAddr, Duration ;
        FROM CGILOG ;
        WHERE recno()>=reccount()-lnReccount ;
        INTO Cursor TQuery ;
        ORDER BY time DESC
   
ELSE
   loHTML.HTMLHeader("CGI Error Log Display")
   SELECT time,Request ;
        FROM CGILOG ;
        WHERE ATC("PROCESSING ERROR",Request)>0 ;
        INTO Cursor TQuery ;
        ORDER BY time DESC
   loHTML.SendLn("<b>CGILOG.DBF Error Count: "+STR(_TALLY)+"</b><p>")

ENDIF

loHTML.ShowCursor(,,.t.)     
loHTML.HTMLFooter()

USE IN TQuery

RETURN

************************************************************************
PROCEDURE ClearLog
******************
***  Modified: 01/12/96
***  Function: Clears the Access Log (CGILOG.DBF) file of records
***      Pass: loCGI
*************************************************************************
LPARAMETER loCGI
LOCAL loHTML
 
IF !OpenExclusive("CGILOG")
   loHTML=CREATE("wwHTML",loCGI.GetOutFile())
   loHTML.HTMLError("CGILOG file was not cleared.",;
                    "Unable to get exclusive use of the file...",;
                    loCGI.GetServerAdmin(),loCGI.GetPreviousUrl())
ELSE
   USE

   IF !USED("LOGBACK")
      IF !FILE("LOGBACK.DBF")
         CREATE TABLE LOGBACK  ;
              (    TIME        T ,;
                 REQUEST     M ,;
                 REMOTEADDR  M ,;
                 DURATION    N (5,2))
      ELSE
         USE LOGBACK          
      ENDIF
      
      SELE LOGBACK
    ENDIF

    APPEND FROM CGILOG
    USE
      
    USE CGILOG EXCLUSIVE
    ZAP
    USE
      
    DO ShowLog with loCGI
ENDIF

RETURN



************************************************************************
PROCEDURE ErrorHandler
**********************
***  Modified: 12/31/95
***  Function: Global Error handler. Shuts down everything then
***            restarts the server from scratch. Used to make sure the
***            server is always running.
***
***            Note that local error handlers in the Process class
***            or other classes will override the global handler.
*************************************************************************
PARAMETER t_errorno,t_mess1,t_mess2,t_module,t_lineno
LOCAL llOldLogging

ON ERROR
llOldLogging=oCGIServer.GetLogToFile()
oCGIServer.SetLogToFile(.T.)
oCGIServer.LogEntry("Processing Error<br>"+CR+;
                    oCGIServer.oCGI.GetCGIParameter()+CR+CR+;
                    "<PRE>"+CR+;
                    "Error: "+STR(T_errorno)+CR+;
                    "Message: "+T_Mess1+CR+;
                    "Code: "+T_mess2+CR+;
                    "Program: "+T_Module+CR+;
                    "Line No: "+STR(T_lineno)+CR+"</PRE>",;
                    "Local",0)

oCGIServer.SetLogToFile(llOldLogging)
CLOSE DATA
RELEASE WINDOWS  && Release CGI Server Window and moves code on

*** Restart - ie. hit the main loop
RETURN TO MASTER

************************************************************************
PROCEDURE Shutit
******************
***  Function: System Shutdown Procedure
*************************************************************************
ON ERROR *
RELEASE oCGIServer
CLEAR ALL
QUIT
RETURN
