************************************************************************
*PROCEDURE FormWiz
******************
***  Modified: 03/02/96
***  Function: Builds an HTML form from a Visual FoxPro form.
***
***    Assume: Form is built using Scalemode = 3 (FOXELS)
***            Font of all objects used is non-proportional.
***            Fields are 'rounded' down to whole integer locations
***            which translate into lines on a form.
***            HTML Forms display using the <PRE> tag.
***
***            Very Preliminary
***
***      Pass: lcFormName  -   Name of the Form to run from
***            lcAction    -   Form ACTION to take when SUBMIT is pressed
***
***    Return: nothing - creates HTM file with same name as form
*************************************************************************
LPARAMETER  lcForm, lcAction, llBindFormFields
LOCAL lcAction, loHTML, lcOutput, lcCaption, lnLastCol, lcWidth

lcAction=IIF(type("lcAction")="C",lcAction,"wwcgi.exe?ProcessForm")
lcForm=IIF(type("lcForm")="C",lcForm,GETFILE('SCX', 'Select Form to Convert:', 'Open',0))

IF EMPTY(lcForm) OR (!FILE(lcForm) AND !FILE(lcForm+".scx"))
   wait window "Form file doesn't exist..." nowait
   RETURN
ENDIF

*** Strip off SCX extension if passed
lcForm=STRTRAN(lower(lcForm),".scx","")
lcFormAlias=SUBSTR(lcForm,RAT("\",lcForm)+1)

#INCLUDE WCONNECT.H

#IF SHAREWARE
  DO WCONNECT.APP
#ELSE  
  SET PROCEDURE TO HTML ADDITIVE
  SET PROCEDURE TO WWUTILS ADDITIVE
#ENDIF  

*** Select all controls into cursor
*** ordered by row and col coords
SELECT *, ;
     INT(VAL( PropStr("Top")) ) AS Top,;
     INT(VAL( PropStr("Left")) ) AS Left ;
  FROM (lcForm+".scx");
  WHERE !EMPTY(PARENT) ;
  ORDER BY TOP, LEFT ;
  INTO DBF HTMLBuild 

*** Retrieve Form Caption
SELECT * ;
  FROM (lcForm+".scx") ;
  WHERE PADR(BASECLASS,4)="form" ;
  INTO CURSOR HTMLForm
lcFormTitle=PropStr("Caption")
USE
  
USE IN (lcFormAlias)


loHTML=CREATE("wwHTML",lcForm+".HTM")

*** Create an empty HTML Header
loHTML.HTMLHeader(lcFormTitle,,,"")  

loHTML.SendLn([<PRE>])
loHTML.SendLn([<FORM ACTION="]+lcAction+[" METHOD="POST">])

lnLastRow=0
lnLastCol=0

SELECT HTMLBuild
SCAN
   lnRow=HTMLBuild.Top
   lnCol=HTMLBuild.Left
     
   IF lnRow#lnLastRow
      lnLastCol=0

      *** Insert Blank Space
      FOR x=1 to lnRow-lnLastRow
         loHTML.SendLn()
      ENDFOR && x=1 to lnRow-lnLastRow
      lnLastRow=lnRow
   ENDIF  
   
   *** Figure out how much space to display
   lnColSpaces=lnCol-lnLastCol
   IF lnColSpaces>0
     loHTML.Send( SPACE(lnColSpaces) )
   ELSE
     *** Always force at least one space!
     loHTML.Send(" ")
   ENDIF
  
   lnLastCol=lnLastCol+lnColSpaces 
      
   DO CASE
      CASE UPPER(BaseClass) = "LABEL"
         *** Use Caption to display text on output
         lcOutput=STRTRA(PropStr("Caption"),"\<","")
         lnLastCol=lnLastCol+LEN(lcOutput)
         lcOutput="<b>"+lcOutput+"</b>"
      CASE UPPER(BaseClass) = "TEXTBOX"
         lcWidth=ALLTRIM(PropStr("Width"))
         lcValue=IIF(llBindFormFields and !EMPTY(PropStr("ControlSource")),;
                     "##"+PropStr("ControlSource")+"##",;
                     PropStr("Value",.t.) )
         lcWidth=IIF(ATC(".",lcWidth)>0,LEFT(lcWidth,ATC(".",lcWidth)-1),lcWidth)
         lcOutput=[<INPUT TYPE="TEXT" NAME="]+PropStr("Name")+[" VALUE="]+lcValue+[" SIZE="]+lcWidth+[">]
         lnLastCol=lnLastCol+Val(lcWidth)
      CASE UPPER(BaseClass) = "EDITBOX"
         lcWidth=ALLTRIM( STR(VAL(ALLTRIM(PropStr("Width")))))
         lcHeight=ALLTRIM(STR(VAL(ALLTRIM(PropStr("Height")))))
         lcWidth=IIF(ATC(".",lcWidth)>0,LEFT(lcWidth,ATC(".",lcWidth)-1),lcWidth)
         lcOutput=[<TEXTAREA  NAME="]+PropStr("Name")+[" COLS="]+lcWidth+[" ROWS="]+lcHeight+[">]+;
                  PropStr("Value",.T.)+;
                  [</TEXTAREA>]+CR
         lnLastCol=lnLastCol+Val(lcWidth)
         lnLastRow=lnLastRow+VAL(lcHeight)

       CASE UPPER(BaseClass) = "COMMANDBUTTON"
         lcCaption=STRTRAN(PropStr("Caption"),"\<","")
         lcWidth=ALLTRIM(PropStr("Width"))
         lcWidth=IIF(ATC(".",lcWidth)>0,LEFT(lcWidth,ATC(".",lcWidth)-1),lcWidth)
         lcOutput=[<INPUT TYPE="]+IIF(PropStr("Cancel")=".T.","RESET","SUBMIT")+[" VALUE="]+lcCaption+[" SIZE="]+lcWidth+[">]  
         lnLastCol=lnLastCol+Val(lcWidth)
       CASE UPPER(BaseClass) = "CHECKBOX"
         lcWidth=ALLTRIM(STR(LEN(PropStr("Caption"))+2))
         lcWidth=IIF(ATC(".",lcWidth)>0,LEFT(lcWidth,ATC(".",lcWidth)-1),lcWidth)

         lcOutput=[<INPUT TYPE="CHECKBOX" ]+IIF(PropStr("Value")=".T." OR ALLTRIM(PropStr("Value"))="1","CHECKED","")+[ SIZE="]+lcWidth+[">]+PropStr("Caption")  

         lnLastCol=lnLastCol+Val(lcWidth)
       
       OTHERWISE
         lcOutput=""
   ENDCASE   

   loHTML.Send(lcOutput)
ENDSCAN

loHTML.Sendln(CR+[</FORM>]+CR+[<PRE>])

loHTML.HTMLFooter

RELEASE loHTML

IF GoUrl(FULLPATH(lcForm+".htm"))=2
   MODI COMM (lcForm+".htm")
ENDIF   

RETURN

  
************************************************************************
PROCEDURE PropStr
******************
***  Function: Extracts a Property value as a string from an
***            SCX file.
***      Pass: tcProperty  -  Name of the property value to return
***            tlNoError   -  Don't embed error string on invalid
***                           property
***    Return: Property value  or "Error String" or "" if tlNoError 
*************************************************************************
LParameter tcProperty, tlNoError
LOCAL lcRetVal
lcRetVal=Extract(Properties,CHR(10)+tcProperty+" = ",CHR(13))
IF EMPTY(lcRetVal)
   *** Try again without the CHR(10) at the front
   lcRetVal=Extract(Properties,tcProperty+" = ",CHR(13))
ENDIF
   
RETURN IIF(!EMPTY(lcRetVal),STRTRAN(lcRetVal,'"',""),;
           IIF(tlNoError,"","*-- Invalid Property: "+tcProperty+" --**"))
